/*
 *  ngtk/c/ngtk_scroll.c
 *
 *  Copyright (C) 1998, 1999 Gabriel Fort <gabx@tfz.net>
 *
 *  Some parts of this code are inspired by the SLang library's scrolling
 *  code, Copyright (C) 1992, 1995 John E. Davis
 *
 *  This code is ugly and unoptimized, so it's quite awfull to see in
 *  in action... I'm gonna fix it when I've the time.
 *  
 */

/*----------------------------------------------------------------------------

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the 
    Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
    Boston, MA  02111-1307  USA.

----------------------------------------------------------------------------*/
#include <ngtk/ngtk.h>

ngtk_scline_t *ngtk_mk_scroll_line(char *str, int padding)
{
    int padlen = padding - strlen(str);
    int realen;
    char *buf = 0;
    ngtk_scline_t *scline = (ngtk_scline_t *)malloc(sizeof(ngtk_scline_t));

    if (padlen < 0) {
        padlen = 0;
        realen = padding;
    } else
        realen = strlen(str);
    buf = (char *)malloc(padlen + 1);

    if (!scline)
        return(NULL);

    memset((char *)scline, sizeof(ngtk_scline_t), 0);

    if (padlen) {
        memset(buf, 32, padlen);
        buf[padlen] = 0;
    } else
        strcpy(buf, "\0");

    scline->str = (char *)malloc(padding+1);
    strncpy(scline->str, str, realen);
    scline->str[realen] = 0;
    strncat(scline->str, buf, padding);
    free(buf);

    if (!scline->str) {
        free(scline);
        return(NULL);
    }

    return(scline);
}

void ngtk_free_scroll_line(ngtk_scline_t *line)
{
    free(line->str);
    free(line);
}

ngtk_scroll_t *ngtk_mk_scroll_buffer(ngtk_win_t *container, int nlines, \
                                     char *strs[nlines+1])
{
    ngtk_scroll_t *buffer = (ngtk_scroll_t *)malloc(sizeof(ngtk_scroll_t));
    int winlen;
    int i;

    buffer->container = container;
    buffer->nlines = nlines;
    buffer->cur_index = 0;
    winlen = buffer->container->win->_maxx+1;

    buffer->lines = (ngtk_scline_t **)malloc((nlines + 1) * sizeof(ngtk_scline_t *));

    for (i=0;i<=nlines-1;i++) {
        buffer->lines[i] = ngtk_mk_scroll_line(strs[i], winlen);
        if (i)
            buffer->lines[i]->prev = buffer->lines[i-1];
        else
            buffer->lines[i]->prev = NULL;
    }
    for (i=0;i<=nlines-1;i++)
        if (i<nlines-1)
            buffer->lines[i]->next = buffer->lines[i+1];
        else
            buffer->lines[i]->next = NULL;

    return(buffer);
}

void ngtk_free_scroll_buffer(ngtk_scroll_t *buffer)
{
    int i;

    for (i=0;i<=buffer->nlines-1;i++)
        ngtk_free_scroll_line(buffer->lines[i]);

    free(buffer);
}

ngtk_scroll_t *ngtk_mk_scroll_win(int height, int width, int top, int left,
                                  int cpair, int attr, int echo_cursor,
                                  int shadow, int nlines, char *strs[nlines+1])
{
    ngtk_scroll_t *buffer;
    ngtk_win_t *container;

    container = ngtk_push_win(height, width, top, left, cpair, attr, echo_cursor, shadow);
    ngtk_clrwin(container->win, 32, 0);
    buffer = ngtk_mk_scroll_buffer(container, nlines, strs);

    return(buffer);
}

void ngtk_display_scroll_buffer(ngtk_scroll_t *buffer, int topline)
{
    int i;
    int y = buffer->container->win->_maxy;

    for (i=topline;i<=topline+y;i++)
        mvwaddstr(buffer->container->win, i-topline, 0, buffer->lines[i]->str);
    wnoutrefresh(buffer->container->win);

}

void ngtk_scroll_line_up(ngtk_scroll_t *buffer)
{
    if ((buffer->cur_index-1) >= 0) {
        buffer->cur_index--;
        ngtk_display_scroll_buffer(buffer, buffer->cur_index);
    }
}

void ngtk_scroll_line_down(ngtk_scroll_t *buffer)
{
    if ((buffer->cur_index+2+buffer->container->win->_maxy) <= buffer->nlines) {
        buffer->cur_index++;
        ngtk_display_scroll_buffer(buffer, buffer->cur_index);
    }
}
